#-------------------------------------------------------------------------------
# Name:        Axeda Thermostat Cloud Demo
# Purpose:     To illistrait how to use the Axeda Cloud AT commands to
#              implement a basic thermostat.
#
# Author:      sep
#
# Created:     27/05/2014
# Copyright:   (c) Inventek Systems, LLC. 2014
#
# Name:        INVENTEK WIRELESS INTEROPERABILITY NETWORK (IWIN), AT Command Software
# Purpose:     To provide Inventek customers Python scripts illustrating how to
#              use the IWIN AT Command Software.
#
#------------------------------------------------------------------------------
#LICENSE:
#   THE IWIN PROGRAM IS FREE SOFTWARE FOR INVENTEK CUSTOMERS ONLY.
#   INVENTEK SYSTEMS RETAINS ALL OWNERSHIP AND INTELLECTUAL PROPERTY RIGHTS
#   IN THE IWIN CODE ACCOMPANYING THIS MESSAGE AND IN ALL DERIVATIVES HERETO.
#   USER MAY USE THE IWIN CODE, AND ANY DERIVATIVES CREATED BY ANY PERSON OR
#   ENTITY BY OR ON USER?S BEHALF, EXCLUSIVELY WITH INVENTEK'S PROPRIETARY
#   PRODUCTS.
#
#   USER AGREES THAT USER IS SOLELY RESPONSIBLE FOR TESTING THE CODE AND
#   DETERMINING IT?S SUITABILITY.  INVENTEK HAS NO OBLIGATION TO MODIFY, TEST,
#   CERTIFY, OR SUPPORT THE CODE.
#
#   USER ACCEPTANCE AND/OR USE OF THE IWIN CODE CONSTITUTES AGREEMENT TO THE
#   TERMS AND CONDITIONS OF THIS NOTICE. IWIN CAN BE REDISTRIBUTED AND/OR
#   MODIFIED BY INVENTEK CUSTOMERS ONLY AND UNDER THE TERMS OF THE GNU GENERAL
#   PUBLIC LICENSE AS PUBLISHED BY THE FREE SOFTWARE FOUNDATION, EITHER VERSION
#   3 OF THE LICENSE, OR (AT USER?S OPTION) ANY LATER VERSION.
#
#   THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT WITHOUT
#   ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF MERCHANTABILITY OR FITNESS
#   FOR A PARTICULAR PURPOSE.  REFERENCE THE GNU GENERAL PUBLIC LICENSE FOR MORE
#   DETAILS.
#
#   CODE ACCOMPANYING THIS MESSAGE IS SUPPLIED BY INVENTEK "AS IS".  NO WARRANTIES,
#   WHETHER EXPRESSED, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED TO,
#   IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A
#   PARTICULAR PURPOSE APPLY TO THIS CODE, IT?S INTERACTION WITH INVENTEK 'S
#   PRODUCTS, COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN ANY APPLICATION.
#
#   USER ACKNOWLEDGES AND AGREES THAT, IN NO EVENT, SHALL INVENTEK BE LIABLE,
#   WHETHER IN CONTRACT, WARRANTY, TORT (INCLUDING NEGLIGENCE OR BREACH OF
#   STATUTORY DUTY), STRICT LIABILITY, INDEMNITY, CONTRIBUTION, OR OTHERWISE,
#   FOR ANY INDIRECT, SPECIAL, PUNITIVE, EXEMPLARY, INCIDENTAL OR CONSEQUENTIAL
#   LOSS, DAMAGE, FOR COST OR EXPENSE OF ANY KIND WHATSOEVER RELATED TO THE CODE,
#   HOWSOEVER CAUSED, EVEN IF INVENTEK HAS BEEN ADVISED OF THE POSSIBILITY OR THE
#   DAMAGES ARE FORESEEABLE.  TO THE FULLEST EXTENT ALLOWABLE BY LAW, INVENTEK 'S
#   TOTAL LIABILITY ON ALL CLAIMS IN ANY WAY RELATED TO THIS CODE, SHALL NOT
#   EXCEED THE PRICE PAID DIRECTLY TO INVENTEK SPECIFICALLY TO HAVE THIS CODE
#   DEVELOPED.
#
#   USER SHOULD HAVE RECEIVED A COPY OF THE GNU GENERAL PUBLIC LICENSE ALONG
#   WITH THIS PROGRAM. IF NOT, SEE <HTTP://WWW.GNU.ORG/LICENSES/>.
#-------------------------------------------------------------------------------
#
# Version:      0.0         sep         20140527    Intial release
#               0.1         sep         20140613    Added Usage instructiuons
#               0.2         sep         20140801    Added varirables to make setup easier
#
# Usage:        1.  Set useSoftAP to 1 to use SoftAP for Configuration or to
#                   0 to configure manually (see Join Network section)
#               2.  Set the Protocol and Remote IP address (see Setup and Start
#                   TCP Client section)
#               3.  Determine if your server will echo back what it has been sent?
#                   Reccommend: http://www.hw-group.com/products/hercules/index_en.html
#                   The TCP and UDP servers can be set to echo back what they receive
#                   Set serverEcho to 1 if echoing, O if not.
#               4.  Plug in a EVK board (Please note will have needed to install
#                   drivers for the board using the eS-WiFi Demo software)
#               5.  Run the script
#
#-------------------------------------------------------------------------------

import math, sys, serial, serial.tools.list_ports, time

#Quiet mode, 0 = Quiet, 1 = Verbose(See all AT commands)
quiet = 1

#Send timeout
timeout = 30


#Send a command to a eS-WiFi module and wait for prompt("\r\n> ") responce
def sendCMDBlocking(serPort,cmd):
    rcvString =""

    #Send Command
    if (quiet == 1):
        sys.stdout.write(cmd)
    serPort.write(cmd + '\r')

    st = time.time()                    #Get time

    #read until "\r\n> " received
    found = 0
    while found != 1:
        for c in serPort.read():
            rcvString += c
            if  '\r\n> ' in rcvString:
                found = 1
            #Endif
        #End for

        #Check for timeout
        if (time.time() - st) > timeout:
            return '-1'                 #Timeout error
        #Endif
    #End while

    #Show responce
    if (quiet == 1):
        sys.stdout.write(rcvString)

    #Get payload if OK responce
    if 'OK\r\n> ' in rcvString:
        rcvString=rcvString[2:(len(rcvString)-8)]
    #Endif

    #Return payload or Error
    return rcvString


#Send data to a eS-WiFi module and wait for prompt("\r\n> ") responce
#   Using a byte array with the S3 command embedded in array
#   e.g. S3=100\r<100 bytes of data>
def sendDataBlocking(serPort,data):
    rcvString =""

    #Send Command
    if (quiet == 1):
        sys.stdout.write(data)
    serPort.write(data)

    st = time.time()                    #Get time

    #read until "\r\n> " received
    found = 0
    while found != 1:
        for c in serPort.read():
            rcvString += c
            if  '\r\n> ' in rcvString:
                found = 1
            #Endif
        #End for

        #Check for timeout
        if (time.time() - st) > timeout:
            return '-1'                 #Timeout error
        #Endif
    #End while

    #Show responce
    if (quiet == 1):
        sys.stdout.write(rcvString)

    #Get payload if OK responce
    if 'OK\r\n> ' in rcvString:
        rcvString=rcvString[2:(len(rcvString)-8)]
    #Endif

    #Return payload or Error
    return rcvString

#Send CMD and wait for responce, waiting nsec seconds before timing out
def sendCMDWait(serPort,cmd,responce,nsec):
    rcvString =""

    #Send Command
    if (quiet == 1):
        sys.stdout.write(cmd)
    serPort.write(cmd + '\r')

    st = time.time()                    #Get time

    #Check each message recieved for responce
    found = 0
    while found != 1:

        #read until "\r\n> " received
        found0 = 0
        while found0 != 1:
            for c in serPort.read():
                rcvString += c
                #if (quiet == 1):
                sys.stdout.write(c)
                if  "\r\n> " in rcvString:
                    found0 = 1
                #Endif
            #End for

            #Check for timeout
            if (time.time() - st) > nsec:
                return '-1'                 #Timeout error
            #Endif
        #End while

        #Show responce
        if (quiet == 1):
            sys.stdout.write(rcvString)

        #Check for responce
        if  responce in rcvString:
            found = 1
        else:
            rcvString = ""                  #Clear string
        #End if
    #End while

    #Show responce
    if (quiet == 1):
        sys.stdout.write(rcvString)

    #Get payload if OK responce
    if 'OK\r\n> ' in rcvString:
        rcvString=rcvString[2:(len(rcvString)-8)]
    #Endif

    #Return payload or Error
    return rcvString


#Search COM port for and eS-WiFi module
def findModule(timeout):
    rcvString =""

    #Get ports
    portList = list(serial.tools.list_ports.comports())
    #print(portList)

    #Search for module
    for index, openPort in enumerate(portList):

        #Serial Port Setup
        port = portList[index][0]
        baud = 115200
        serPort = serial.Serial(port,baud)
        serPort.timeout = 10

        serPort.write('\r')                 #Send CR

        st = time.time()                    #Get time

        #read until "\r\n> " received
        found = 0
        while found != 1:
            for c in serPort.read():
                rcvString += c
                if  '\r\n> ' in rcvString:
                    found = 1
                #Endif
            #End for

            #Check for timeout
            if (time.time() - st) > timeout:
                return '-1'                 #Timeout error
            #Endif
        #End while

        #Try to get info
        rcv = sendCMDBlocking(serPort,"I?")

        #Close serial port
        serPort.close()

        #Found?
        if (found == 1):
            #print("Port = " + port)
            if "ISM4" in rcv:
                print(rcv)
                return port
            #Endif
        #End if
    #End for
    return -1

#Application
def main():

    #Redirect std err to std out
    sys.stderr = sys.stdout

    #Serial Port Setup
    port = findModule(10)
    if (port == -1):
        print( "Failed to find an eS-WiFi module!")
        sys.exit(-1)
    #End if
    baud = 115200
    ser = serial.Serial(port,baud)
    ser.timeout = 10

    bd = 115200                     #Default
    #Choose baud rate for demo (uncomment choice)
    #bd = 115200
    #bd = 460800
    #bd = 921600
    bd = 1382400
    #bd = 1843200
    #bd = 2073600
    #bd = 3916800

    #Get Firmware information
    if (quiet == 1):
        rcv = sendCMDBlocking(ser,"I?")
        print(rcv)
    #End if

    #Setup ---------------------------------------------------------------------
    mCnt = 910                      #Monitoring Count
    nob = 1152                      #Number of bytes to send
                                    #910 * 1152 = 1048320 ~(1MB)

    PROTOCOL=0                      #TCP=0, TCP-SSL=3

    useSoftAP = 0                   #Join using, 0 = Cx commands, 1 = A0 (SoftAP)
    remoteIP = "192.168.1.7"        #Remote Server IP address
    remotePort = "8002"             #Remote TCP port
    SSID = "ssid"                   #SSID of AP
    PSWD = "password"               #Password of AP
    SEC="4"                         #Security type of AP: 0-Open, 1-WEP, 2-WPA, 3-WPA2-AES, 4-WPA2-Mixed
    DHCP="1"                        #Get IP address for DHCP, 0-No, 1-Yes

    serverEcho = 1                  #Server is going to echo data back, 0=No, 1=Yes
    chgBaudRate = 1                 #Enable change baud rate, 0=Disable, 1=Enable
    printData = 0                   #Print send and receive data, 0=No, 1=Yes
   #---------------------------------------------------------------------------

    monitorCount = 0            #Monitor counter

    #Start Application
    print( "eS-WiFi TCP Client Demo:" )

    if chgBaudRate == 1:
        rcv = sendCMDBlocking(ser, "U2=" + str(bd))     #Set Baud
        ser.write("U0\r")                               #Start
        time.sleep(.1)                                  #Wait 100ms
        ser.close()                                     #Close
        ser = serial.Serial(port,bd)                    #Reopen
        rcv = sendCMDBlocking(ser, "U?")                #UART Info
        print( "Communications: " + ser.portstr + "@" + str(bd))
    else:
        print( "Communications: " + ser.portstr + "@" + str(baud))
    print( "Packet Size:    " + str(nob) + " bytes")
    print( "" )


    #Setup data
    data = bytearray("S3=" + str(nob) + "\r")
    cmpData = bytearray()
    for i in range(0, nob):
        data.append(255 - (i % 0x100))
        cmpData.append(255 - (i % 0x100))
    #End for


    #Join Network
    if (useSoftAP == 0):
        rcv = sendCMDBlocking(ser, "CS")                        #Is connected?
        if (rcv == '0'):
            rcv = sendCMDBlocking(ser, "C1="+SSID)             #Set SSID
            rcv = sendCMDBlocking(ser, "C2="+PSWD)              #Set Password
            rcv = sendCMDBlocking(ser, "C3="+SEC)               #Security Type: 0-Open, 1-WEP, 2-WPA, 3-WPA2-AES, 4-WPA2-Mixed
            rcv = sendCMDBlocking(ser, "C4="+DHCP)              #Use DHCP

            #Get start time
            st = time.time()

            #Join
            rcv = sendCMDBlocking(ser, "C0")                    #Join network
            if (rcv != '-1'):
                if "ERROR" in rcv:                              #Check for error
                    print( "Join failed to connect to network\r\n> ")
                    sys.exit(-1)
                else:
                    print( "Time: " + ("%012.3F" % (time.time()-st)) + " Joining: " + rcv[10:] )
                #End if
            else:
                print( "Joined timed out" )
                sys.exit(-1)
            #End if
        else:
            print( "Already connected to network" )
        #Endif
    else:
        rcv = sendCMDBlocking(ser, "CS")                        #Is connected?
        if (rcv == '1'):
            rcv = sendCMDBlocking(ser, "CD")                    #Dis-connected

        #SoftAP
        rcv = sendCMDBlocking(ser, "Z5")                        #Get MAC address
        print( "Starting Configuration Access Point, look for TCP_DEMO_" + rcv )

        #Setup SoftAP
        rcv = sendCMDBlocking(ser, "AS=1,TCP_DEMO")             #Set SoftAP SSID

        #Start and wait for Join message
        rcv = sendCMDWait(ser, "A0","[JOIN   ]",300)            #Start SoftAP
        if (rcv != '-1'):                                       #Check for timeout
            if "ERROR" in rcv:                                  #Check for error
                print( "Join failed to connect to network")
                sys.exit(-1)
            else:
                print( "Time: " + ("%012.3F" % (time.time()-st)) + " Joining: " + rcv[10:] )
            #End if
        else:
            print( "Timeout, failed to connect to a network")
            serPort.write(ctrlQ)                                #Send Quit
            sys.exit(-1)
        #End if
    #End if

    #Configure LEDs
    rcv = sendCMDBlocking(ser, "G4=3,1")        #Red LED
    rcv = sendCMDBlocking(ser, "G4=4,1")        #Green LED

    #Green - OFF, RED - OFF
    rcv = sendCMDBlocking(ser, "G3=0,1,0")      #Red LED
    rcv = sendCMDBlocking(ser, "G3=1,1,0")      #Green LED

    #Setup and Start TCP Client
    rcv = sendCMDBlocking(ser, "P1="+PROTOCOL)      #TCP/TCP-SSL
    rcv = sendCMDBlocking(ser, "P3="+remoteIP)      #Remote IP
    rcv = sendCMDBlocking(ser, "P4="+remotePort)    #Port
    rcv = sendCMDBlocking(ser, "P9=0")              #No certifcate verification
    rcv = sendCMDBlocking(ser, "P6=1")              #Start Client

    #rcv = sendCMDBlocking(ser, "R2=5000")           #Read timeout

    #Get start time
    print( "Sending..." )
    st = time.time()

    #Reporting Loop
    while monitorCount < mCnt:

        rcv0 = sendDataBlocking(ser, data)           #Send data
        if serverEcho == 1:
            rcv1 = bytearray( sendCMDBlocking(ser, "R0") )        #Read echoed data
            if printData == 1:
                print ("Sent Data:")
                print ''.join('{:02x}'.format(x) for x in cmpData)
                print ("Received Data:")
                print ''.join('{:02x}'.format(x) for x in rcv1)
            #Endif
            if cmpData != rcv1:
                print("Error: Mis-match")
            else:
                print("Passed")
            #Endif
        #Endif

        monitorCount += 1                       #Monitor counter
    #End while

    #Get start time
    et = time.time()
    print( "Complete" )

    #Leave Network
    rcv = sendCMDBlocking(ser, "CD")            #Leave network

    #Green - OFF, RED - OFF
    rcv = sendCMDBlocking(ser, "G3=0,1,0")      #Red LED
    rcv = sendCMDBlocking(ser, "G3=1,1,0")      #Green LED

    #Statistics
    print( "" )
    print( "Summary:" )
    print( "Total time(sec) = " + str(et-st))
    print( "Total bytes TX  = " + str(monitorCount * nob))
    if serverEcho == 1:
        print( "Total bytes RX  = " + str(monitorCount * nob))
    #Endif
    print( "Total bytes/sec = " + str( (monitorCount * nob) / (et-st) ) )
    if chgBaudRate == 1:
        print( "Total bits/sec =  " + str( ((monitorCount * nob) / (et-st)) * 8 ) ) + ", Theoretical = " + str( (bd/10) * 8 )
    else:
        print( "Total bits/sec =  " + str( ((monitorCount * nob) / (et-st)) * 8 ) )


    #Reset Baud rate
    if chgBaudRate == 1:
        bd = 115200
        rcv = sendCMDBlocking(ser, "U2=" + str(bd))         #Set Baud
        ser.write("U0\r")                                   #Start
        time.sleep(.1)                                      #Wait 100ms
        ser.close()                                         #Close
        ser = serial.Serial(port,bd)                        #Reopen
        rcv = sendCMDBlocking(ser, "U?")                    #UART Info
        #print(rcv)

    #Close serial port
    ser.close()

if __name__ == '__main__':
    sys.exit(main())

